@@ -6,6 +6,8 @@
 #include <stdlib.h>
 #include <pwd.h>
 #include <errno.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 
 #include "mt.h"
 #include "error.h"
@@ -18,6 +20,7 @@
 #include "misc.h"
 #include "globals.h"
 
+int scrollback_search_to_new_window(buffer *pbuf, char *org_title, char *find_str, mybool_t case_insensitive);
 
 int find_string(buffer *pbuf, char *find, int offset, char direction, mybool_t case_insensitive)
 {
@@ -100,21 +103,24 @@ void scrollback_savefile(buffer *pbuf)
 			{
 				if ((pbuf -> be)[loop].Bline)
 				{
-					char display;
+					char display = 1;
 					char *error;
 					int dummy = -1;
 					regmatch_t *pmatch = NULL;
 
 					/* check filter */
-					(void)check_filter((pbuf -> be)[loop].pi, (pbuf -> be)[loop].Bline, &pmatch, &error, &dummy, 0, &display);
-					if (error)
+					if (!IS_MARKERLINE((pbuf -> be)[loop].pi))
 					{
-						fprintf(fh, "%s\n", error);
-						myfree(error);
+						(void)check_filter((pbuf -> be)[loop].pi, (pbuf -> be)[loop].Bline, &pmatch, &error, &dummy, 0, &display);
+						if (error)
+						{
+							fprintf(fh, "%s\n", error);
+							myfree(error);
+						}
 					}
 					if (display)
 					{
-						fprintf(fh, "%s\n", (pbuf -> be)[loop].Bline?(pbuf -> be)[loop].Bline:"");
+						fprintf(fh, "%s\n", USE_IF_SET((pbuf -> be)[loop].Bline, ""));
 					}
 
 					if (pmatch) myfree(pmatch);
@@ -142,23 +148,30 @@ int get_lines_needed(char *string, int terminal_width)
 
 void scrollback_displayline(int window_nr, NEWWIN *win, buffer *pbuf, int buffer_offset, int terminal_offset, int offset_in_line, mybool_t force_to_winwidth, char show_winnr)
 {
-	proginfo *cur_line_meta = (pbuf -> be)[buffer_offset].pi;
 	char *cur_line = (pbuf -> be)[buffer_offset].Bline;
-	double ts = (pbuf -> be)[buffer_offset].ts;
-	regmatch_t *pmatch = NULL;
-	int matching_regex = -1;
-	char *error = NULL;
 
 	wmove(win -> win, terminal_offset, 0);
 
 	if (cur_line)
 	{
+		proginfo *cur_line_meta = (pbuf -> be)[buffer_offset].pi;
+		double ts = (pbuf -> be)[buffer_offset].ts;
+		char old_color_settings = 0;
+
+		if (scrollback_no_colors && cur_line_meta != NULL)
+		{
+			old_color_settings = cur_line_meta -> cdef.colorize;
+			cur_line_meta -> cdef.colorize = 0;
+		}
 		if (IS_MARKERLINE(cur_line_meta))
 		{
 			color_print(window_nr, win, cur_line_meta, cur_line, NULL, -1, MY_FALSE, 0, 0, ts, show_winnr);
 		}
 		else /* just a buffered line */
 		{
+			char *error = NULL;
+			regmatch_t *pmatch = NULL;
+			int matching_regex = -1;
 			char display;
 			(void)check_filter(cur_line_meta, cur_line, &pmatch, &error, &matching_regex, 0, &display);
 			if (error)
@@ -183,30 +196,60 @@ void scrollback_displayline(int window_nr, NEWWIN *win, buffer *pbuf, int buffer
 					color_print(window_nr, win, cur_line_meta, cur_line, pmatch, matching_regex, force_to_winwidth, 0, 0, ts, show_winnr);
 				}
 			}
+
+			myfree(pmatch);
+		}
+		if (scrollback_no_colors && cur_line_meta != NULL)
+		{
+			cur_line_meta -> cdef.colorize = old_color_settings;
 		}
 	}
 	else /* an empty line */
 	{
 		/* do nothing */
 	}
-
-	myfree(pmatch);
 }
 
-void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
+int scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 {
+	int rc = 0;
 	char *find = NULL;
 	NEWWIN *mywin1, *mywin2;
 	int nlines = max_y - 6, ncols = max_x - 6;
-	int offset = max(0, pbuf -> curpos - nlines);
+	int offset = max(0, pbuf -> curpos - nlines); // FIXME: aantal regels laten afhangen van lengte
 	char redraw = 2;
 	int line_offset = 0;
 	char show_winnr = default_sb_showwinnr;
 	mybool_t case_insensitive = re_case_insensitive;
+	buffer cur_lb;
+	int loop;
 
+	memset(&cur_lb, 0x00, sizeof(cur_lb));
+
+	for(loop=0; loop<pbuf -> curpos; loop++)
+	{
+		if ((pbuf -> be)[loop].Bline == NULL)
+			continue;
+
+		cur_lb.be = myrealloc(cur_lb.be, (cur_lb.curpos + 1) * sizeof(buffered_entry), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+		cur_lb.be[cur_lb.curpos].pi    = (pbuf -> be)[loop].pi;
+		if ((pbuf -> be)[loop].pi != NULL && (pbuf -> be)[loop].pi -> cdef.term_emul != TERM_IGNORE)
+		{
+			color_offset_in_line *cmatches;
+			int n_cmatches;
+			cur_lb.be[cur_lb.curpos].Bline = emulate_terminal((pbuf -> be)[loop].Bline, &cmatches, &n_cmatches);
+			myfree(cmatches);
+		}
+		else
+			cur_lb.be[cur_lb.curpos].Bline = strdup((pbuf -> be)[loop].Bline);
+		cur_lb.be[cur_lb.curpos].ts    = (pbuf -> be)[loop].ts;
+		cur_lb.curpos++;
+	}
+
+	LOG("---\n");
 	if (global_highlight_str)
 	{
-		find = mystrdup(global_highlight_str, "scrollback_do: global highlight str");
+		find = mystrdup(global_highlight_str, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	}
 
 	mywin1 = create_popup(max_y - 4, max_x - 4);
@@ -224,7 +267,7 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			int lines_used = 0;
 
 			ui_inverse_on(mywin1);
-			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", header, pbuf -> curpos);
+			mvwprintw(mywin1 -> win, nlines + 1, 1, "%s - %d buffered lines", shorten_filename(header, max(24, ncols - 24)), cur_lb.curpos);
 			ui_inverse_off(mywin1);
 
 			if (!no_linewrap) ui_inverse_on(mywin1);
@@ -233,18 +276,46 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 
 			werase(mywin2 -> win);
 
-			for(;(offset + index) < pbuf -> curpos;)
+			if (!no_linewrap && line_offset > 0)
 			{
-				int prev_lines_used = lines_used;
+				int temp_line_offset = line_offset;
+				int n_chars_to_display_left = strlen((cur_lb.be)[offset].Bline) - temp_line_offset;
+
+				while(lines_used < nlines && n_chars_to_display_left > 0)
+				{
+					scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, &cur_lb, offset, lines_used, temp_line_offset, 1, show_winnr);
+
+					temp_line_offset += ncols;
+					n_chars_to_display_left -= ncols;
 
-				if (no_linewrap)
 					lines_used++;
+				}
+
+				index++;
+			}
+
+			for(;(offset + index) < cur_lb.curpos && lines_used < nlines;)
+			{
+				int lines_needed = get_lines_needed((cur_lb.be)[offset + index].Bline, ncols);
+
+				if (no_linewrap || lines_needed == 1)
+				{
+					scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, &cur_lb, offset + index, lines_used, no_linewrap?line_offset:0, no_linewrap, show_winnr);
+					lines_used++;
+				}
 				else
-					lines_used += get_lines_needed((pbuf -> be)[offset + index].Bline, ncols);
-				if (lines_used > nlines)
-					break;
+				{
+					int cur_line_offset = 0;
+
+					while(lines_used < nlines && lines_needed > 0)
+					{
+						scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, &cur_lb, offset + index, lines_used, cur_line_offset, 1, show_winnr);
+						cur_line_offset += ncols;
+						lines_used++;
+						lines_needed--;
+					}
+				}
 
-				scrollback_displayline(winnrs?winnrs[offset + index]:window_nr, mywin2, pbuf, offset + index, prev_lines_used, line_offset, no_linewrap, show_winnr);
 				index++;
 			}
 
@@ -261,10 +332,15 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 		c = wait_for_keypress(HELP_SCROLLBACK_HELP, 0, NULL, 1);
 		uc = toupper(c);
 
-		if (uc == 'Q' || uc == 'X' || c == abort_key || c == KEY_CLOSE || c == KEY_EXIT)
+		if (c == 'q' || uc == 'X' || c == abort_key || c == KEY_CLOSE || c == KEY_EXIT)
 		{
 			break;
 		}
+		else if (c == 'Q' || c == -1)		/* Q: close whole stack of scrollbackwindows, -1: something got closed */
+		{
+			rc = -1;
+			break;
+		}
 		else if (c == 20 && winnrs != NULL)	/* ^t */
 		{
 			show_winnr = 1 - show_winnr;
@@ -281,7 +357,7 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			statistics_menu();
 		}
 		else if ((c == KEY_LEFT ||
-			  c == KEY_BACKSPACE)
+					c == KEY_BACKSPACE)
 				&& no_linewrap)
 		{
 			if (line_offset > 0)
@@ -332,25 +408,27 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 'k' ||
 					/* c == 11  || *//* ^k */
 					c == 16)    /* ^p */
-				&& offset > 0)
+				&& (offset > 0 || (!no_linewrap && line_offset > 0)))
 		{
-			int n_to_move = 0;
-
-			wmove(mywin2 -> win, 0, 0);
-
-			do
+			if (no_linewrap)
+			{
+				offset--;
+			}
+			else if (line_offset > 0)
+			{
+				line_offset = max(0, line_offset - ncols);
+			}
+			else
 			{
 				offset--;
 
-				if (no_linewrap)
-					n_to_move = 1;
-				else
-					n_to_move = get_lines_needed((pbuf -> be)[offset].Bline, ncols);
-			} while(n_to_move == 0 && offset > 0);
+				line_offset = (get_lines_needed((cur_lb.be)[offset].Bline, ncols) - 1) * ncols;
+			}
 
-			winsdelln(mywin2 -> win, n_to_move);
+			wmove(mywin2 -> win, 0, 0);
+			winsdelln(mywin2 -> win, 1);
 
-			scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, pbuf, offset, 0, line_offset, no_linewrap, show_winnr);
+			scrollback_displayline(winnrs?winnrs[offset]:window_nr, mywin2, &cur_lb, offset, 0, line_offset, no_linewrap, show_winnr);
 
 			redraw = 1;
 		}
@@ -361,14 +439,25 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 14  || /* ^n */
 					c == 13  ||
 					c == KEY_ENTER)
-				&& offset < (pbuf -> curpos - 1))
+				&& offset < (cur_lb.curpos - 1))
 		{
-			int n_to_move = 0;
-
-			while(n_to_move == 0 && offset < (pbuf -> curpos - 1))
+			if (no_linewrap)
 			{
 				offset++;
-				n_to_move = get_lines_needed((pbuf -> be)[offset].Bline, ncols);
+			}
+			else if (strlen((cur_lb.be)[offset].Bline) > (line_offset + ncols))
+			{
+				line_offset += ncols;
+			}
+			else if (offset < (cur_lb.curpos - 1))
+			{
+				if (strlen((cur_lb.be)[offset].Bline) > (line_offset + ncols))
+					line_offset += ncols;
+				else
+				{
+					line_offset = 0;
+					offset++;
+				}
 			}
 
 			redraw = 2;
@@ -381,24 +470,45 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 'z' ||
 					c == 'u' ||
 					c == ('U' - 65 + 1))   /* ^u */
-				&& offset < (pbuf -> curpos - 1))
+				&& offset < (cur_lb.curpos - 1))
 		{
-			int dummy = 0;
-			int lines = 0;
-			int page_size = (c == 'u' || c == ('U' - 65 + 1)) ? (nlines / 2) : nlines;
-			while(offset < (pbuf -> curpos - 1))
+			if (no_linewrap)
 			{
-				offset++;
-				dummy += get_lines_needed((pbuf -> be)[offset].Bline, ncols);
-				if (dummy > page_size && c == KEY_NPAGE)
+				offset += nlines;
+				if (offset >= cur_lb.curpos)
+					offset = cur_lb.curpos - 1;
+			}
+			else
+			{
+				int n_lines_to_move = nlines;
+
+				while(n_lines_to_move > 0 && offset < (cur_lb.curpos))
 				{
-					if (lines > 0)
-						offset--;
-					break;
+					if (line_offset > 0)
+					{
+						if (line_offset + ncols >= strlen((cur_lb.be)[offset].Bline))
+						{
+							line_offset = 0;
+							offset++;
+							n_lines_to_move--;
+						}
+						else
+						{
+							line_offset += ncols;
+							n_lines_to_move--;
+						}
+					}
+					else
+					{
+						n_lines_to_move -= get_lines_needed((cur_lb.be)[offset].Bline, ncols);
+						offset++;
+					}
 				}
 
-				lines++;
+				if (n_lines_to_move < 0)
+					line_offset = (-n_lines_to_move) * ncols;
 			}
+
 			redraw = 2;
 		}
 		else if ((c == KEY_PPAGE ||
@@ -409,40 +519,50 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 					c == 4)         /* ^d */
 				&& offset > 0)
 		{
-			int dummy = 0;
-			int lines = 0;
-			int page_size = (c == 'd' || c == 4) ? (nlines / 2) : nlines;
-			while(offset > 0)
+			if (no_linewrap)
 			{
-				offset--;
-				dummy += get_lines_needed((pbuf -> be)[offset].Bline, ncols);
-				if (dummy > page_size)
-				{
-					if (lines > 0)
-						offset++;
-					break;
-				}
-
-				lines++;
+				offset -= nlines;
+				if (offset < 0)
+					offset = 0;
 			}
+			else
+			{
+				int n_lines_to_move = nlines;
+
+				if (line_offset)
+					n_lines_to_move -= line_offset / ncols;
+
+				while(n_lines_to_move > 0 && offset > 0)
+				{
+					offset--;
+
+					n_lines_to_move -= get_lines_needed((cur_lb.be)[offset].Bline, ncols);
+
+					if (n_lines_to_move < 0)
+					{
+						line_offset = (get_lines_needed((cur_lb.be)[offset].Bline, ncols) + n_lines_to_move) * ncols;
+					}
+				}
+			}
+
 			redraw = 2;
 		}
 		else if ((c == KEY_HOME ||
-			  c == 'g' ||
-			  c == '<' ||
-			  c == KEY_SBEG)
+					c == 'g' ||
+					c == '<' ||
+					c == KEY_SBEG)
 				&& offset > 0)
 		{
 			line_offset = offset = 0;
 			redraw = 2;
 		}
 		else if ((c == KEY_END ||
-			  c == 'G' ||
-			  c == '>' ||
-			  c == KEY_SEND)
-				&& offset < (pbuf -> curpos - 1))
+					c == 'G' ||
+					c == '>' ||
+					c == KEY_SEND)
+				&& offset < (cur_lb. curpos - 1))
 		{
-			offset = pbuf -> curpos - 1;
+			offset = cur_lb. curpos - 1;
 			redraw = 2;
 		}
 		else if (uc == 'R' || c == ('R' - 65 + 1) || c == ('L' - 65 + 1) || c == KEY_REFRESH)
@@ -483,22 +603,35 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 
 			if (find)
 			{
-				int new_f_index;
-
-				redraw = 2; /* force redraw */
-
-				regfree(&global_highlight_re);
-				myfree(global_highlight_str);
-				global_highlight_str = NULL;
-
-				new_f_index = find_string(pbuf, find, 0, direction, case_insensitive);
-				if (new_f_index == -1)
+				if (scrollback_search_new_window)
 				{
-					wrong_key();
+					if (scrollback_search_to_new_window(&cur_lb, header, find, case_insensitive) == -1)
+					{
+						/* cascaded close */
+						rc = -1;
+						break;
+					}
 				}
 				else
 				{
-					offset = new_f_index;
+					int new_f_index;
+
+					redraw = 2; /* force redraw */
+
+					regfree(&global_highlight_re);
+					myfree(global_highlight_str);
+					global_highlight_str = NULL;
+
+					new_f_index = find_string(&cur_lb, find, 0, direction, case_insensitive);
+					if (new_f_index == -1)
+					{
+						wrong_key();
+					}
+					else
+					{
+						offset = new_f_index;
+						line_offset = 0;
+					}
 				}
 			}
 		}
@@ -508,7 +641,7 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 			{
 				char direction = (c == 'n' || c == KEY_NEXT) ? 1 : -1;
 				int start_offset = offset + direction;
-				int new_f_index = find_string(pbuf, find, start_offset, direction, case_insensitive);
+				int new_f_index = find_string(&cur_lb, find, start_offset, direction, case_insensitive);
 				if (new_f_index == -1)
 				{
 					wrong_key();
@@ -517,6 +650,7 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 				{
 					redraw = 2; /* force redraw */
 					offset = new_f_index;
+					line_offset = 0;
 				}
 			}
 			else
@@ -526,7 +660,7 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 		}
 		else if (c == 's' || c == KEY_SAVE)
 		{
-			scrollback_savefile(pbuf);
+			scrollback_savefile(&cur_lb);
 			redraw = 2;	/* force redraw */
 		}
 		else if (c == 'h')
@@ -561,6 +695,10 @@ void scrollback_do(int window_nr, buffer *pbuf, int *winnrs, char *header)
 	delete_popup(mywin1);
 
 	myfree(find);
+
+	delete_be_in_buffer(&cur_lb);
+
+	return rc;
 }
 
 void scrollback(void)
@@ -581,7 +719,7 @@ void scrollback(void)
 	if (window != -1 && lb[window].bufferwhat != 0)
 	{
 		int header_size = strlen(pi[window].filename) + 4;
-		char *header = (char *)mymalloc(header_size + 1, "window header");
+		char *header = (char *)mymalloc(header_size + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 		snprintf(header, header_size, "%02d] %s", window, pi[window].filename);
 		scrollback_do(window, &lb[window], NULL, header);
 		myfree(header);
@@ -591,7 +729,7 @@ void scrollback(void)
 void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 {
 	int lc_size = nfd * sizeof(int);
-	int *last_check = (int *)mymalloc(lc_size, "last check index");
+	int *last_check = (int *)mymalloc(lc_size, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 	int *winnr = NULL;
 	buffer buf;
 	regex_t reg;
@@ -622,13 +760,10 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 		char *string;
 		int checked_all = 0;
 
-		LOG("chosen_ts: %f\n", chosen_ts);
-
 		for(loop=0; loop<nfd; loop++)
 		{
 			if (lb[loop].curpos == last_check[loop])
 			{
-				LOG("curpos is last_check\n");
 				checked_all++;
 				continue;
 			}
@@ -640,7 +775,6 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 				/* did not match? don't add and continue */
 				if (rc == REG_NOMATCH)
 				{
-					LOG("reg nomatch %d / %d: %s\n", loop, last_check[loop], lb[loop].be[last_check[loop]].Bline);
 					continue;
 				}
 
@@ -652,10 +786,8 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 				}
 			}
 
-			LOG("string found: %s\n", lb[loop].be[last_check[loop]].Bline);
 			if (lb[loop].be[last_check[loop]].ts <= chosen_ts)
 			{
-				LOG("~~~ early\n");
 				chosen_ts = lb[loop].be[last_check[loop]].ts;
 				chosen_win = loop;
 			}
@@ -663,8 +795,6 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 
 		if (chosen_win == -1)
 		{
-			LOG("not found\n");
-
 			if (checked_all == nfd)
 				break;
 
@@ -677,18 +807,16 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 			continue;
 		}
 
-		LOG("add!");
-
 		if (!IS_MARKERLINE(lb[chosen_win].be[last_check[chosen_win]].pi))
 		{
 			/*** ADD LINE TO BUFFER ***/
-			buf.be = (buffered_entry *)myrealloc(buf.be, sizeof(buffered_entry) * (buf.curpos + 1), "merged_scrollback_with_search: buffered_entry");
-			winnr  = (int *)myrealloc(winnr, sizeof(int) * (buf.curpos + 1), "merged_scrollback_with_search: winnr");
+			buf.be = (buffered_entry *)myrealloc(buf.be, sizeof(buffered_entry) * (buf.curpos + 1), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			winnr  = (int *)myrealloc(winnr, sizeof(int) * (buf.curpos + 1), __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			curline = buf.curpos++;
 			/* add the logline itself */
 			string = lb[chosen_win].be[last_check[chosen_win]].Bline;
 			if (string)
-				buf.be[curline].Bline = mystrdup(string, "merged_scrollback_with_search: print_and_buffer");
+				buf.be[curline].Bline = mystrdup(string, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			else
 				buf.be[curline].Bline = NULL;
 			/* remember pointer to subwindow (required for setting colors etc.) */
@@ -711,13 +839,13 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 		{
 			char *help = "Searched for: ";
 			int len = strlen(help) + strlen(search_for) + 1;
-			header = mymalloc(len, "merged_scrollback_with_search header (1)");
+			header = mymalloc(len, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			snprintf(header, len, "%s%s", help, search_for);
 		}
 		else
 		{
 			char *help = "Merge view";
-			header = mymalloc(strlen(help) + 1, "merged_scrollback_with_search header (2)");
+			header = mymalloc(strlen(help) + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
 			sprintf(header, "%s", help);
 		}
 
@@ -729,4 +857,51 @@ void merged_scrollback_with_search(char *search_for, mybool_t case_insensitive)
 	delete_be_in_buffer(&buf);
 
 	myfree(winnr);
+
+	myfree(last_check);
+}
+
+int scrollback_search_to_new_window(buffer *pbuf, char *org_title, char *find_str, mybool_t case_insensitive)
+{
+	int loop, rc;
+	regex_t regex;
+	buffer cur_lb;
+	char *new_header;
+
+	/* compile the searchstring (which can be a regular expression) */
+	if ((rc = regcomp(&regex, find_str, REG_EXTENDED | (case_insensitive == MY_TRUE?REG_ICASE:0))))
+	{
+		regexp_error_popup(rc, &regex);
+		return 0;
+	}
+
+	memset(&cur_lb, 0x00, sizeof(buffer));
+
+	for(loop=0; loop<pbuf -> curpos; loop++)
+	{
+		if ((pbuf -> be)[loop].Bline == NULL)
+			continue;
+
+		if (regexec(&regex, (pbuf -> be)[loop].Bline, 0, NULL, 0) == 0)
+		{
+			cur_lb.be = myrealloc(cur_lb.be, (cur_lb.curpos + 1) * sizeof(buffered_entry), __FILE__, __PRETTY_FUNCTION__, __LINE__);
+			cur_lb.be[cur_lb.curpos].Bline = (pbuf -> be)[loop].Bline;
+			cur_lb.be[cur_lb.curpos].pi    = (pbuf -> be)[loop].pi;
+			cur_lb.be[cur_lb.curpos].ts    = (pbuf -> be)[loop].ts;
+			cur_lb.curpos++;
+		}
+	}
+
+	new_header = (char *)mymalloc(strlen(org_title) + 1 + strlen(find_str) + 1, __FILE__, __PRETTY_FUNCTION__, __LINE__);
+	sprintf(new_header, "%s %s", org_title, find_str);
+
+	rc = scrollback_do(-1, &cur_lb, NULL, new_header);
+
+	myfree(new_header);
+
+	myfree(cur_lb.be);
+
+	regfree(&regex);
+
+	return rc;
 }
